// FIX LAMM trading example
// 
// This source file is intended to be a simple and concise example
// which demonstrates how to send orders to accounts under a LAMM acccount.
// The process involves getting a record of all accounts under the LAMM
// by reading in the account fields from CollateralReport messages, and then
// by sending an NewOrderList which contains order requests for each
// account 
//
// This example assumes that you have some familiarity with the FIX protocol.
// All messages which are routinely used by a FIX application are not demonstrated here.
// Only the most relevant messages to the example are used. 
//

#include <iostream>
#include <time.h>
#include <vector>

#include "quickfix\Application.h"
#include "quickfix\FileLog.h"
#include "quickfix\FileStore.h"
#include "quickfix\fix44\CollateralInquiry.h"
#include "quickfix\fix44\CollateralReport.h"
#include "quickfix\fix44\NewOrderList.h"
#include "quickfix\fix44\NewOrderSingle.h"
#include "quickfix\fix44\TradingSessionStatus.h"
#include "quickfix\fix44\TradingSessionStatusRequest.h"
#include "quickfix\fix44\UserRequest.h"
#include "quickfix\fix44\UserResponse.h"
#include "quickfix\MessageCracker.h"
#include "quickfix\Session.h"
#include "quickfix\SessionSettings.h"
#include "quickfix\SocketInitiator.h"

using namespace std;
using namespace FIX;

class FixGateway : public MessageCracker, public Application
{
private:
	// Used to signal that all accounts under the LAMM have been accounted for
	bool accounts_collected_;
	// The integer variable which is incremented and used to create request IDs
	int request_id_;
	// Local session id which is required when sending messages over the API
	SessionID session_id_;
	// Copy of SessionSettings. In our application we use this to retrieve values
	// such as login credentials and TargetSubID
	SessionSettings gateway_settings_;
	// Request ID for the CollateralInquiry message. We want this variable to be
	// in scope for the life of the application. This allows us to make a 
	// CollateralInquiry request and check CollateralReport messages against 
	// this ID to ensure a specific reponse was solicited by us and not
	// automatically generated by the API
	string collinquiry_id_;
	// Used to create a collection of account numbers for which we will
	// place orders
	vector<string> account_list_;

	// FIX fields which are FXCM specific. For a complete list of FXCM specific flields, 
	// see the FIX Specification PDF available at 
	// http://forexforums.dailyfx.com/fix-api-support/453300-fix-documentation-subscribe-updates.html
	enum FXCM_FIX_FIELDS
	{
		LIST_OF_TRADING_SESSION = 5
	};
public:
	// FixGatweay implements the FIX Application class. At a minimum we want to construct this
	// class with the location of SessionSettings. This settings class allows us to retrieve
	// information such as login credentials
	FixGateway(SessionSettings settings)
	{
		request_id_ = 0;
		accounts_collected_ = false;
		gateway_settings_ = settings;
	}

	// Gets called when quickfix creates a new session. A session comes into and remains in existence
	// for the life of the application. Sessions exist whether or not a counter party is connected to it. 
	// As soon as a session is created, you can begin sending messages to it. If no one is logged on, 
	// the messages will be sent at the time a connection is established with the counterparty.
	void onCreate(const SessionID& sessionid)
	{
		session_id_ = sessionid;
		cout << "Session created with ID " << sessionid << endl;
	}

	// Notifies you when a valid logon has been established with a counter party. This is called when 
	// a connection has been established and the FIX logon process has completed with both parties 
	// exchanging valid logon messages. 
	void onLogon(const SessionID& sessionid)
	{
		cout << "Session logged on with ID " << sessionid << endl;
		// Now the the session has logged on, we want to authenticate ourselves over the API.
		// We do this by sending a UserRequest message. Note that if you do not do this first,
		// other messages will be rejected. Below we create this message, fill the required fields,
		// and then send it over the API.
		FIX44::UserRequest request;
		request.setField(FIX::UserRequestID(NextId())); // Unique identifier for the UserRequest message 
		request.setField(FIX::Username(gateway_settings_.get().getString("Username"))); // Lamm login
		request.setField(FIX::Password(gateway_settings_.get().getString("Password"))); // Lamm password
		request.setField(FIX::UserRequestType(LIST_OF_TRADING_SESSION)); // FXCM defined field, always 5
		FIX::Session::sendToTarget(request,sessionid);
	}

	// Notifies you when an FIX session is no longer online. This could happen during a normal logout
	// exchange or because of a forced termination or a loss of network connection. 
	void onLogout(const SessionID& sessionid)
	{
		cout << "Session logged out with ID " << sessionid << endl;
	}

	// Provides you with a peak at the administrative messages that are being sent from your FIX engine 
	// to the counter party. This is normally not useful for an application however it is provided for 
	// any logging you may wish to do. Notice that the FIX::Message is not const. This allows you to add
	// fields before an adminstrative message before it is sent out.
	void toAdmin(Message& message, const SessionID& sessionid)
	{
		// FXCM requires the TargetSubID field in the header message. Here we get this value
		// from the settings file and set the appropriate field in the FIX header message
		string target_sub_id = gateway_settings_.get().getString("TargetSubID");
		message.getHeader().setField(FIX::TargetSubID(target_sub_id));
	}

	// A callback for application messages that you are being sent to a counterparty. If you throw a
	// DoNotSend exception in this function, the application will not send the message. This is mostly useful
	// if the application has been asked to resend a message such as an order that is no longer relevant for
	// the current market. Messages that are being resent are marked with the PossDupFlag in the header set to
	// true; If a DoNotSend exception is thrown and the flag is set to true, a sequence reset will be sent in
	// place of the message. If it is set to false, the message will simply not be sent. Notice that the FIX::Message
	// is not const. This allows you to add fields before an application message before it is sent out.
	void toApp(Message& message, const SessionID& sessionid)
	{
		// FXCM requires the TargetSubID field in the header message. Here we get this value
		// from the settings file and set the appropriate field in the FIX header message
		string target_sub_id = gateway_settings_.get().getString("TargetSubID");
		message.getHeader().setField(FIX::TargetSubID(target_sub_id));
	}

	// Notifies you when an administrative message is sent from a counterparty to your FIX engine. This can be useful
	// for doing extra validation on logon messages such as for checking passwords. Throwing a RejectLogon exception
	// will disconnect the counterparty.
	void fromAdmin(const Message& message, const SessionID& sessionid)
	{
		// Call the MessageCracker::crack() method which will send the FIX::Message to the appropriate
		// overloaded onMessage() function below
		crack(message,sessionid);
	}

	// One of the core entry points for your FIX application. Every application level request will come through here. 
	// If, for example, your application is a sell-side OMS, this is where you will get your new order requests. If you
	// were a buy side, you would get your execution reports here. If a FieldNotFound exception is thrown, the counterparty
	// will receive a reject indicating a conditionally required field is missing. The Message class will throw this exception
	// when trying to retrieve a missing field, so you will rarely need the throw this explicitly. You can also throw an
	// UnsupportedMessageType exception. This will result in the counterparty getting a reject informing them your application
	// cannot process those types of messages. An IncorrectTagValue can also be thrown if a field contains a value that is out
	// of range or you do not support.
	void fromApp(const Message& message, const SessionID& sessionid)
	{
		// Call the MessageCracker::crack() method which will send the FIX::Message to the appropriate
		// overloaded onMessage() function below
		crack(message,sessionid);
	}

	// This message is used to respond to a user request message, it reports the status of the user after the completion
	// of any action requested in the user request message.
	void onMessage(const FIX44::UserResponse& response, const SessionID& sessionid)
	{
		// Get the UserStatus field from the UserResponse message. If the user status is equal to
		// "1", then we were properly authenticated
		string status = response.getField(FIX::FIELD::UserStatus);
		if(!status.compare("1"))
		{
			// We were properly authenticated. We now want to request TradingSessionStatus to
			// ensure the market is open and available for trading.
			cout << "User authenticated" << endl;
			FIX44::TradingSessionStatusRequest request;
			request.setField(FIX::TradSesReqID(NextId())); // Unique identifier for the TradingSessionStatusRequest message
			request.setField(FIX::SubscriptionRequestType(FIX::SubscriptionRequestType_SNAPSHOT)); // Type of request; either snapshot or subscription
			FIX::Session::sendToTarget(request,sessionid);
		}
		else
		{
			cout << "Could not authenticate user" << endl;
		}
	}

	// The Trading Session Status provides information on the status of a market. We use this primarily to determine
	// if the Session is open and available for trading. 
	void onMessage(const FIX44::TradingSessionStatus& session_status, const SessionID& sessionid)
	{
		// Get the TradSesStatus field from the message. If this value is equal to "2", the market is
		// open and available for trading 
		string status = session_status.getField(FIX::FIELD::TradSesStatus);
		if(!status.compare("2"))
		{
			// Trading is open, we now want to request the list of accounts under the LAMM. 
			// Initialize a new request id for the CollateralInquiry message. This CollateralInquiry
			// message is a request for the CollateralReport, a message containing each account
			// under the LAMM
			collinquiry_id_ = NextId();

			FIX44::CollateralInquiry inquiry;
			inquiry.setField(FIX::CollInquiryID(collinquiry_id_)); // The unqiue identifier for the CollateralInquiry
			inquiry.setField(FIX::SubscriptionRequestType(FIX::SubscriptionRequestType_SNAPSHOT)); // The type of request, either snapshot or subscription
			FIX::Session::sendToTarget(inquiry,sessionid);
		}
		else
		{
			cout << "Trading is closed" << endl;
		}
	}

	// Used to report collateral status when responding to a Collateral Inquiry message.
	void onMessage(const FIX44::CollateralReport& report, const SessionID& sessionid)
	{
		// The code below will read each CollateralReport message for the account it
		// contains. Each new account will be added to our account_list_ variable.

		// Get the account field from the CollateralReport. 
		string account = report.getField(FIX::FIELD::Account);
		// If the account_list_ size is currently zero, we have never added even one
		// account. Therefore, add this account to account_list_
		if((int)account_list_.size() == 0)
		{
			account_list_.push_back(account);
			cout << "Added " << account << " to list of accounts being managed" << endl;
		}
		// If the account_list_ size is greater than zero, we want to check and see if
		// the account from the current CollateralReport has already been added. If so,
		// we will do nothing.
		else
		{
			// For each account number in account_list_ ...
			for(int i = 0; i < (int)account_list_.size(); i++)
			{
				// If we find that the account field from this CollateralReport is equal 
				// to the value at this position in account_list_, then break
				if(account == account_list_.at(i))break;
				// If we have iterated through account_list_ entirely and have not found the
				// account value contained, then add it
				if(i == (int)account_list_.size() - 1)
				{
					account_list_.push_back(account);
					cout << "Added " << account << " to list of accounts being managed" << endl;
				}
			}
		}
		// The code below checks to see if we have accounted for all accounts under the LAMM
		// We check to see if the CollInquiryID field is set. This field is set when the CollateralReport
		// being returned is the result of a CollateralInquiry request. If it is set and its value
		// equals our last request ID, then we want to check if all accounts have been accounted for.
		// We do this by checking to see if the total number of CollateralReport messages being returned
		// is equal to the size of our account_list_. 
		if(report.isSetField(FIX::FIELD::CollInquiryID))
		{
			// If the CollInquiryID equals our last request ID of this type ...
			if(!report.getField(FIX::FIELD::CollInquiryID).compare(collinquiry_id_))
			{
				// Get the total number of CollateralReports being returned and compare this against
				// the size of our account_list_
				int report_count = FIX::IntConvertor::convert(report.getField(FIX::FIELD::TotNumReports));
				if((int)account_list_.size() == report_count)
				{
					cout << "All accounts collected" << endl;
					// Set our flag variable to true which signals that orders can now
					// be sent
					accounts_collected_ = true;
				}
			}
		}
	}

	// Used to respond to a Collateral Inquiry in the following situations:
	// 1. When the CollateralInquiry will result in an out of band response (such as a file transfer).
	// 2. When the inquiry is otherwise valid but no collateral is found to match the criteria specified 
	// on the Collateral Inquiry message.
	// 3. When the Collateral Inquiry is invalid based upon the business rules of the counterparty.
	void onMessage(const FIX44::CollateralInquiryAck& ack, const SessionID& sessionid)
	{

	}

	// The execution report message is used to:
	// 1. confirm the receipt of an order
	// 2. confirm changes to an existing order (i.e. accept cancel and replace requests)
	// 3. relay order status information
	// 4. relay fill information on working orders
	// 5. relay fill information on tradeable or restricted tradeable quotes
	// 6. reject orders
	void onMessage(const FIX44::ExecutionReport& report, const SessionID& sessionid)
	{

	}

	// The Position Report message is returned by the holder of a position in response to a Request for 
	// Position message. The purpose of the message is to report all aspects of a position and may be 
	// provided on a standing basis to report end of day positions to an owner.
	void onMessage(const FIX44::PositionReport& report, const SessionID& sessionid)
	{

	}

	// Creates a string equivalent of an incremented unsigned int. This string is used as the unique
	// identifier for newly created messages being sent to the API
	string NextId()
	{
		// Check to ensure the request_id_ value is near the max size for an
		// unsigned int. We also account for the potential size of the millisecond value
		// as we increment each request id by this amount (to ensure uniqueness). If
		// the request id nears this max value, reset to 0
		if(request_id_ == (2147483647 - 1000))request_id_ = 0;
		request_id_++;
		// Get local system time
		SYSTEMTIME st;
		GetLocalTime(&st);
		// Increment the request_id_ by the millisecond value to ensure uniqueness
		int request = request_id_ + st.wMilliseconds;
		// Return the string equivalent of request_id_. The FIX::IntConvertor::convert() method
		// can be used to convert between int and string types
		return FIX::IntConvertor::convert(request);
	}

	// Accessor for the accounts_collected_ bool variable. This value is checked in
	// the main thread to wait for sending orders
	bool AccountsCollected()
	{
		return accounts_collected_;
	}

	// This is the central method to our example. This method sends the NewOrderList message
	// to the API. NewOrderList is essentially a map that contains multiple order messages.
	// These order messages may be associated with different accounts under the LAMM. You will
	// notice below that one of the required fields to each order message is the account field.
	// In our example, we will create one order for each account under the LAMM. We have a full
	// list of accounts now that our CollateralReport messages have been processed.
	void BatchOrderRequest
		(
		string symbol, // The symbol to trade; e.g., EUR/USD
		int trade_size, // The trade size (order quantity); e.g., 10,000
		const char trade_side // The side of the trade, buy or sell
		)
	{
		FIX44::NewOrderList order_list; // The message map containing each order message
		order_list.setField(FIX::ListID(NextId())); // Unique identifier for the NewOrderList message

		// We must specify the total number of orders expected in the NewOrderList message. Set
		// this value to the size of account_list_ given that each account under the LAMM
		// will be receiveing an order
		order_list.setField(FIX::TotNoOrders((int)account_list_.size()));
		// Now we must map an order to the NewOrderList for each account in account_list_
		// Each iteration below will be used to set the account field of an order
		for(int i = 0; i < (int)account_list_.size(); i++)
		{
			// Create new instance of the NewOrderList::NoOrders group. This is a FIX repeating
			// group class to which order details are mapped
			FIX44::NewOrderList::NoOrders order;
			order.setField(FIX::ListSeqNo(i)); // The sequence of the order in the list
			order.setField(FIX::ClOrdID(NextId())); // Unique identifier for the order in the list
			order.setField(FIX::Account(account_list_.at(i))); // The account at this position in account_list_
			order.setField(FIX::Symbol(symbol)); // The symbol to trade; e.g., EUR/USD
			order.setField(FIX::OrderQty(trade_size)); // The trade size; e.g., 10,000
			order.setField(FIX::Side(trade_side)); // The side of the trade; e.g., Buy or Sell
			order.setField(FIX::TransactTime(FIX::TransactTime())); // The transaction time
			order.setField(FIX::OrdType(FIX::OrdType_MARKET)); // OrderType; i.e., market, limit, stop, etc.

			// Add the newly created NoOrders group to the NewOrderList
			order_list.addGroup(order);
		}
		// Send the list to the API
		FIX::Session::sendToTarget(order_list,session_id_);
	}
};

int main()
{
	// Provide location of settings file and initialize instance of SessionSettings
	SessionSettings settings("settings.cfg");
	// Here we construct FixGateway that implements the Application and MessageCracker classes.
	// The FixGateway class will be the main gateway through which messages are received and sent over the API
	FixGateway gateway(settings);
	// Construct the two classes used for storing FIX messages and writing them to log files
	FileStoreFactory storeFactory(settings);
	FileLogFactory logFactory(settings);
	// Construct the SocketInitiator class which establishes the FIX Session
	SocketInitiator initiator(gateway, storeFactory, settings, logFactory);
	// Start the session
	initiator.start();
	// Wait until all accounts have been collected. When they are, send
	// the batch of order requests to each account under the LAMM
	while(true)
	{
		if(gateway.AccountsCollected())
		{

			cout << "Sending batch order request " << endl;
			gateway.BatchOrderRequest("EUR/USD", 10000, FIX::Side_BUY);
			Sleep(5000);
			break;
		}
	}
	// End the session
	initiator.stop();

	cin.get();
	return 0;
}